@@ -18,14 +18,14 @@
 
 */
 
-/* $Header: /repository/projects/libjsonevt/utf16.h,v 1.2 2009-02-23 17:46:55 don Exp $ */
+/* $Revision: 1568 $ */
 
 #ifndef UTF16_H
 #define UTF16_H
 
-#include "uni.h"
+#include <uni.h>
 
-#include "int_defs.h"
+#include <int_defs.h>
 
 UNI_DO_CPLUSPLUS_WRAP_BEGIN
 
